#!/usr/bin/env node
var moment = require('moment');
var cli = require('cli');
var config = require('./sitegin/config');

cli.main(function(args, options) {
  console.log(options);
  config()
  .then(function(obj) {
    sitegin(obj.config);
  })
  .catch(function(e) {
    console.log(e.stack);
    process.exit();
  });
});

var sitegin = function(config) {
  var options = config.options;
  require('./sitegin/sitegin')({
    watch: !options.noserver
  })
  .then(function onLoad(jobs) {
    var sass = require('node-sass');
    var fs = require('fs');

    // Main builder function
    // If builder is not running - run it
    // If it is running - schedule rerun after it finishes
    var isRunning = false;
    var runAgain = false;
    var doSync = function(){}
    function run() {
      var startTime = moment();
      runAgain = false;
      if(isRunning) {
        console.log("Generator is still running. Queing another run after it finishes.")
        runAgain = true;
        return;
      }
      console.log("================================================================================")
      console.log("Running generator")
      isRunning = true;
      jobs.run('pipeline',jobs)
      .then(function() {
        isRunning = false;
        console.log("Generator finished in",moment().diff(startTime,'seconds'),'seconds')
        doSync();
        if(runAgain) run();
      })
      .catch(function(e) {
        isRunning = false;
        console.log("Generator crashed in",moment().diff(startTime,'seconds'),'seconds')
        if(runAgain) run();
        if(e.stack)
          console.log(e.stack)
        else
          console.log(e)
      });
    }

    console.log('Sitegin successfully loaded');
    run();
    copyStaticFiles(config.builddir);
    rendersass(config.builddir);

    if(!options.noserver) {
      var sync = require('browser-sync').create();
      doSync = function() {
        sync.reload('*')
      }
      sync.init({
        server: {
          baseDir: config.builddir
        },
        ui: {
          port: options.port+1
        },
        port: options.port
      });

      var chokidar = require('chokidar');
      // article or theme reload
      chokidar.watch(['content/','theme/'], {ignoreInitial: true})
      .on('all', function(event, path) {
        console.log('Content or theme changed. Rebuilding...');
        run();
      })

      // sitegin reload
      jobs.onReload(function() {
        console.log("Sitegin reloaded");
        run();
      })
    }
  })
  .catch(function(e) {
    console.log(e.stack);
    process.exit();
  });
}

var copyStaticFiles = function(builddir) {
  // STATIC FILES
  var fsextra = require('node-fs-extra');
  fsextra.copy(
      'static',
      builddir,
      function(file){ return !(file.match(".git") || file.match("static/articles")); },
      function(){console.log("copy static done");}
    );
  fsextra.copy(
      'static/articles',
      builddir+"/clanek",
      function(){console.log("copy static/articles done");}
    );
  fsextra.copy(
      'theme/static',
      builddir,
      function(file){ return !file.match("\\.git") },
      function(){console.log("copy theme/static done");}
    );
}

var rendersass = function(builddir) {
  // SASS
  var sass = require('node-sass');
  var fs = require('fs');

  sass.render ({file: "theme/sass/style.scss"},function(err, result) {
    if(err == null) {
      console.log("compiled sass");
      fs.writeFile(builddir+"/theme/style.css",result.css);
    } else console.log("error ", err);
  });
};
